package de.mcs.tools.sps;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.reflections.Reflections;
import org.reflections.util.ClasspathHelper;
import org.reflections.util.ConfigurationBuilder;

import de.mcs.tools.sps.annotations.SPSOutputter;

public class OutputFactory {

  private static final String DEFAULT_PACKAGE_FILTER = "de.mcs.tools.sps"; //$NON-NLS-1$

  private static Map<String, Outputter> outputters = new HashMap<>();

  static {
    Set<Class<?>> outputClasses = searchOutputter(SPSOutputter.class);
    for (Class<?> outClass : outputClasses) {
      SPSOutputter annotation = outClass.getAnnotation(SPSOutputter.class);
      try {
        Outputter outputter = (Outputter) outClass.getConstructor().newInstance();
        outputters.put(annotation.name().toUpperCase(), outputter);
      } catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException
          | NoSuchMethodException | SecurityException e) {
        e.printStackTrace();
      }
    }

  }

  private static Set<Class<?>> searchOutputter(Class<? extends Annotation> annotationClass) {
    ConfigurationBuilder builder = new ConfigurationBuilder();
    builder.addUrls(ClasspathHelper.forPackage(DEFAULT_PACKAGE_FILTER));
    Reflections reflections = new Reflections(builder);
    Set<Class<?>> classes = new HashSet<Class<?>>();
    classes.addAll(reflections.getTypesAnnotatedWith(annotationClass));
    return classes;
  }

  public static Outputter getOutputter(String outputformat) throws Exception {
    Outputter outputter = outputters.get(outputformat.toUpperCase());
    if (outputter == null) {
      throw new Exception(String.format("can't find outputter for format: %s", outputformat));
    }
    return outputter;
  }
}
